@@ -1,6 +1,6 @@
 /* libpoke.c - Implementation of the public services in libpoke.  */
 
-/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "libpoke.h"
 #include "pk-utils.h"
 #include "pkt.h"
 #include "pkl.h"
@@ -28,7 +29,6 @@
 #include "pkl-env.h" /* XXX */
 #include "pvm.h"
 #include "pvm-val.h" /* XXX */
-#include "libpoke.h"
 #include "ios-dev.h" /* for struct ios_dev_if */
 #include "configmake.h"
 
@@ -38,14 +38,16 @@ struct _pk_compiler
   pvm vm;
 
   int status;  /* Status of last API function call. Initialized with PK_OK */
+  struct ios_dev_if foreign_iod_if;
   /* Data for completion machinery.  */
   pkl_ast_node complete_type;
   ios completion_ios;
   int completion_idx;
   struct pkl_ast_node_iter completion_iter;
+  void *user_data;
 };
 
-struct pk_term_if libpoke_term_if;
+struct pk_term_if_internal libpoke_term_if;
 
 #define PK_RETURN(code) do { return pkc->status = (code); } while (0)
 
@@ -59,7 +61,9 @@ pk_compiler_new_with_flags (struct pk_term_if *term_if, uint32_t flags)
 
   if (!term_if->flush_fn || !term_if->puts_fn || !term_if->printf_fn
       || !term_if->indent_fn || !term_if->class_fn || !term_if->end_class_fn
-      || !term_if->hyperlink_fn || !term_if->end_hyperlink_fn)
+      || !term_if->hyperlink_fn || !term_if->end_hyperlink_fn
+      || !term_if->get_color_fn || !term_if->get_bgcolor_fn
+      || !term_if->set_color_fn || !term_if->set_bgcolor_fn)
     return NULL;
 
   pkc = calloc (1, sizeof (struct _pk_compiler));
@@ -80,7 +84,8 @@ pk_compiler_new_with_flags (struct pk_term_if *term_if, uint32_t flags)
       if (libpoke_datadir == NULL)
         libpoke_datadir = PKGDATADIR;
 
-      libpoke_term_if = *term_if;
+      libpoke_term_if.term_if = *term_if;
+      libpoke_term_if.pkc = pkc;
 
       pkc->vm = pvm_init ();
       if (pkc->vm == NULL)
@@ -131,6 +136,20 @@ pk_errno (pk_compiler pkc)
   return PK_ERROR;
 }
 
+void
+pk_set_user_data (pk_compiler pkc, void *user_data)
+{
+  pkc->status = PK_OK;
+  pkc->user_data = user_data;
+}
+
+void *
+pk_get_user_data (pk_compiler pkc)
+{
+  pkc->status = PK_OK;
+  return pkc->user_data;
+}
+
 int
 pk_compile_file (pk_compiler pkc, const char *filename,
                  pk_val *exit_exception)
@@ -229,9 +248,10 @@ pk_compile_expression (pk_compiler pkc, const char *buffer,
 }
 
 int
-pk_load (pk_compiler pkc, const char *module)
+pk_load (pk_compiler pkc, const char *module, pk_val *exit_exception)
 {
-  PK_RETURN (pkl_load (pkc->compiler, module) == 0 ? PK_ERROR : PK_OK);
+  PK_RETURN (pkl_load (pkc->compiler, module, exit_exception) == 0 ? PK_ERROR
+                                                                   : PK_OK);
 }
 
 void
@@ -241,6 +261,19 @@ pk_set_quiet_p (pk_compiler pkc, int quiet_p)
   pkc->status = PK_OK;
 }
 
+void
+pk_set_debug_p (pk_compiler pkc, int debug_p)
+{
+  pkl_set_debug_p (pkc->compiler, debug_p);
+  pkc->status = PK_OK;
+}
+
+const char *
+pk_get_debug_ast (pk_compiler pkc)
+{
+  return pkl_get_last_ast_str (pkc->compiler);
+}
+
 void
 pk_set_lexical_cuckolding_p (pk_compiler pkc, int lexical_cuckolding_p)
 {
@@ -255,6 +288,13 @@ pk_set_alien_token_fn (pk_compiler pkc, pk_alien_token_handler_fn cb)
   pkc->status = PK_OK;
 }
 
+void
+pk_set_alien_dtoken_fn (pk_compiler pkc, pk_alien_dtoken_handler_fn cb)
+{
+  pkl_set_alien_dtoken_fn (pkc->compiler, (pkl_alien_dtoken_handler_fn) cb);
+  pkc->status = PK_OK;
+}
+
 static char *
 complete_attribute (pk_compiler pkc, const char *x, int state)
 {
@@ -445,35 +485,30 @@ pk_completion_function (pk_compiler pkc,
   return complete_decl (pkc, text, state);
 }
 
-/* This function provides command line completion when the tag of an
-   IOS is an appropriate completion.
-
-   TEXT is the partial word to be completed.  STATE is zero the first
-   time the function is called and non-zero for each subsequent call.
-
-   On each call, the function returns the tag of an IOS for which
-   that tag and TEXT share a common substring. It returns NULL to
-   indicate that there are no more such tags.
- */
 char *
-pk_ios_completion_function (pk_compiler pkc, const char *text, int state)
+pk_ios_completion_function (pk_compiler pkc,
+                            const char *prefix, const char *text, int state)
 {
 #define IO pkc->completion_ios
 
   int len  = strlen (text);
 
-  IO = state == 0 ?  ios_begin () : ios_next (IO);
+  IO = state == 0 ?  ios_begin (pvm_ios_context (pkc->vm)) : ios_next (IO);
   while (1)
     {
+      char *handler = NULL;
+
       if (ios_end (IO))
         break;
 
-      char buf[16];
-      snprintf (buf, 16, "#%d", ios_get_id (IO));
+      handler = pk_str_concat (prefix, ios_get_handler (IO), NULL);
+      if (handler == NULL)
+        return NULL;
 
-      if (strncmp (buf, text, len) == 0)
-        return strdup (buf);
+      if (strncmp (handler, text, len) == 0)
+        return handler;
 
+      free (handler);
       IO = ios_next (IO);
     }
 
@@ -571,14 +606,13 @@ pk_ios
 pk_ios_cur (pk_compiler pkc)
 {
   pkc->status = PK_OK;
-  return (pk_ios) ios_cur ();
+  return (pk_ios) ios_cur (pvm_ios_context (pkc->vm));
 }
 
 void
 pk_ios_set_cur (pk_compiler pkc, pk_ios io)
 {
-  /* XXX use pkc */
-  ios_set_cur ((ios) io);
+  ios_set_cur (pvm_ios_context (pkc->vm), (ios) io);
   pkc->status = PK_OK;
 }
 
@@ -595,19 +629,17 @@ pk_ios_flags (pk_ios io)
 }
 
 pk_ios
-pk_ios_search (pk_compiler pkc, const char *handler)
+pk_ios_search (pk_compiler pkc, const char *handler, uint32_t flags)
 {
   pkc->status = PK_OK;
-  /* XXX use pkc */
-  return (pk_ios) ios_search (handler);
+  return (pk_ios) ios_search (pvm_ios_context (pkc->vm), handler, flags);
 }
 
 pk_ios
 pk_ios_search_by_id (pk_compiler pkc, int id)
 {
   pkc->status = PK_OK;
-  /* XXX use pkc */
-  return (pk_ios) ios_search_by_id (id);
+  return (pk_ios) ios_search_by_id (pvm_ios_context (pkc->vm), id);
 }
 
 int
@@ -615,9 +647,9 @@ pk_ios_open (pk_compiler pkc,
              const char *handler, uint64_t flags, int set_cur_p)
 {
   int ret;
+  ios_context ios_ctx = pvm_ios_context (pkc->vm);
 
-  /* XXX use pkc */
-  if ((ret = ios_open (handler, flags, set_cur_p)) >= 0)
+  if ((ret = ios_open (ios_ctx, handler, flags, set_cur_p)) >= 0)
     return ret;
 
   switch (ret)
@@ -638,8 +670,7 @@ pk_ios_open (pk_compiler pkc,
 void
 pk_ios_close (pk_compiler pkc, pk_ios io)
 {
-  /* XXX use pkc */
-  ios_close ((ios) io);
+  ios_close (pvm_ios_context (pkc->vm), (ios) io);
   pkc->status = PK_OK;
 }
 
@@ -685,8 +716,7 @@ pk_ios_map (pk_compiler pkc,
             pk_ios_map_fn cb, void *data)
 {
   struct ios_map_fn_payload payload = { cb, data };
-  /* XXX use pkc */
-  ios_map (my_ios_map_fn, (void *) &payload);
+  ios_map (pvm_ios_context (pkc->vm), my_ios_map_fn, (void *) &payload);
   pkc->status = PK_OK;
 }
 
@@ -1094,14 +1124,14 @@ pk_print_val (pk_compiler pkc, pk_val val, pk_val *exit_exception)
     pkc->status = PK_OK;
 }
 
-static struct ios_dev_if foreign_iod_if;
-
 int
 pk_register_iod (pk_compiler pkc, struct pk_iod_if *iod_if)
 {
+  ios_context ios_ctx = pvm_ios_context (pkc->vm);
+
   pkc->status = PK_OK;
 
-#define CF(FN) foreign_iod_if.FN = iod_if->FN
+#define CF(FN) pkc->foreign_iod_if.FN = iod_if->FN
   CF (get_if_name);
   CF (handler_normalize);
   CF (open);
@@ -1111,9 +1141,41 @@ pk_register_iod (pk_compiler pkc, struct pk_iod_if *iod_if)
   CF (get_flags);
   CF (size);
   CF (flush);
-  CF (data);
 #undef CF
 
-  (void) ios_register_foreign_iod (&foreign_iod_if);
+  (void) ios_register_foreign_iod (ios_ctx, &pkc->foreign_iod_if, iod_if->data);
   return pkc->status;
 }
+
+int
+pk_keyword_p (pk_compiler pkc, const char *str)
+{
+  static const char *keywords[] =
+    {
+#define PKL_DEF_KEYWORD(s) s,
+# include "pkl-keywords.def"
+#undef PKL_DEF_KEYWORD
+      NULL,
+    };
+  int i;
+
+  for (i = 0; keywords[i] != NULL; ++i)
+    {
+      if (STREQ (str, keywords[i]))
+        return 1;
+    }
+
+  return 0;
+}
+
+void
+pk_register_thread ()
+{
+  pvm_register_thread ();
+}
+
+void
+pk_unregister_thread ()
+{
+  pvm_unregister_thread ();
+}
