@@ -317,7 +317,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_trans1_ps_var)
                                                        NULL /* args */);
 
           PKL_AST_LOC (funcall) = PKL_AST_LOC (var);
-          PKL_PASS_NODE = funcall;
+          PKL_PASS_NODE = ASTREF (funcall);
           PKL_PASS_RESTART = 1;
         }
     }
@@ -1241,20 +1241,21 @@ PKL_PHASE_END_HANDLER
 PKL_PHASE_BEGIN_HANDLER (pkl_trans1_pr_comp_stmt)
 {
   if (PKL_TRANS_FUNCTION)
-    {
-      PKL_TRANS_FUNCTION->back++;
-
-      if (PKL_PASS_PARENT
-          && PKL_AST_CODE (PKL_PASS_PARENT) == PKL_AST_EXP
-          && PKL_AST_EXP_CODE (PKL_PASS_PARENT) == PKL_AST_OP_EXCOND)
-        {
-          PKL_TRANS_FUNCTION->ndrops++;
-          PKL_TRANS_FUNCTION->npopes++;
-        }
-    }
+    PKL_TRANS_FUNCTION->back++;
 
   if (PKL_TRANS_ESCAPABLE)
     PKL_TRANS_ESCAPABLE->nframes++;
+
+  if (PKL_PASS_PARENT
+      && PKL_AST_CODE (PKL_PASS_PARENT) == PKL_AST_EXP
+      && PKL_AST_EXP_CODE (PKL_PASS_PARENT) == PKL_AST_OP_EXCOND)
+    {
+      if (PKL_TRANS_FUNCTION)
+        PKL_TRANS_FUNCTION->npopes++;
+
+      if (PKL_TRANS_ESCAPABLE)
+        PKL_TRANS_ESCAPABLE->npopes++;
+    }
 }
 PKL_PHASE_END_HANDLER
 
@@ -1341,20 +1342,21 @@ PKL_PHASE_BEGIN_HANDLER (pkl_trans1_ps_comp_stmt)
   PKL_AST_COMP_STMT_NUMVARS (comp_stmt) = numvars;
 
   if (PKL_TRANS_FUNCTION)
-    {
-      PKL_TRANS_FUNCTION->back--;
-
-      if (PKL_PASS_PARENT
-          && PKL_AST_CODE (PKL_PASS_PARENT) == PKL_AST_EXP
-          && PKL_AST_EXP_CODE (PKL_PASS_PARENT) == PKL_AST_OP_EXCOND)
-        {
-          PKL_TRANS_FUNCTION->ndrops--;
-          PKL_TRANS_FUNCTION->npopes--;
-        }
-    }
+    PKL_TRANS_FUNCTION->back--;
 
   if (PKL_TRANS_ESCAPABLE)
     PKL_TRANS_ESCAPABLE->nframes--;
+
+  if (PKL_PASS_PARENT
+      && PKL_AST_CODE (PKL_PASS_PARENT) == PKL_AST_EXP
+      && PKL_AST_EXP_CODE (PKL_PASS_PARENT) == PKL_AST_OP_EXCOND)
+    {
+      if (PKL_TRANS_FUNCTION)
+        PKL_TRANS_FUNCTION->npopes--;
+
+      if (PKL_TRANS_ESCAPABLE)
+        PKL_TRANS_ESCAPABLE->npopes--;
+    }
 }
 PKL_PHASE_END_HANDLER
 
@@ -1487,6 +1489,8 @@ PKL_PHASE_BEGIN_HANDLER (pkl_trans1_ps_break_continue_stmt)
         = PKL_TRANS_ESCAPABLE->node;
       PKL_AST_BREAK_CONTINUE_STMT_NFRAMES (PKL_PASS_NODE)
         = PKL_TRANS_ESCAPABLE->nframes;
+      PKL_AST_BREAK_CONTINUE_STMT_NPOPES (PKL_PASS_NODE)
+        = PKL_TRANS_ESCAPABLE->npopes;
     }
 }
 PKL_PHASE_END_HANDLER
@@ -1719,7 +1723,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_trans2_ps_struct_ref)
           PKL_AST_TYPE (funcall) = ASTREF (function_rtype);
           PKL_AST_LOC (funcall) = PKL_AST_LOC (struct_ref);
 
-          PKL_PASS_NODE = funcall;
+          PKL_PASS_NODE = ASTREF (funcall);
           PKL_PASS_RESTART = 1;
         }
     }
@@ -1979,7 +1983,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_trans3_ps_op_sizeof)
   }
 
   pkl_ast_node_free (PKL_PASS_NODE);
-  PKL_PASS_NODE = offset;
+  PKL_PASS_NODE = ASTREF (offset);
   PKL_PASS_RESTART = 1;
 }
 PKL_PHASE_END_HANDLER
